import typing
from datetime import datetime

from django.contrib.syndication.views import Feed
from django.template.loader import render_to_string
from django.urls import reverse

from .models import HomeArticlePage, HomeArticlesPage


class LatestArticlesFeed(Feed):
    def get_object(self, request, id: int) -> HomeArticlesPage:
        return HomeArticlesPage.objects.get(id=id)

    def title(self, obj: HomeArticlesPage) -> str:
        return obj.title

    def link(self, obj: HomeArticlesPage) -> str:
        return obj.get_full_url()

    def items(self, obj: HomeArticlesPage) -> list:
        return HomeArticlePage.objects.live().child_of(obj).order_by("-date")[:32]

    def item_title(self, item: HomeArticlePage) -> str:
        return item.title

    def item_description(self, item: HomeArticlePage) -> str:
        return render_to_string(
            "home/feed_item_description.html",
            {"item": item},
        )

    def item_pubdate(self, item: HomeArticlePage) -> datetime:
        return datetime(
            item.date.year,
            item.date.month,
            item.date.day,
            12,
            0,
        )

    def item_author_name(self, item: HomeArticlePage) -> str:
        if item.author:
            return item.author

        return ""

    def item_categories(self, item: HomeArticlePage) -> list:
        return item.tags.all()

    def item_link(self, item: HomeArticlePage) -> str:
        return item.get_full_url()
